﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.Text;

namespace HeuristicLab.Modeling.SQLiteBackend {  
  [Table(Name="Model")]
  public class Model {
    public Model() {
      targetVariable = default(EntityRef<Variable>);
      algorithm = default(EntityRef<Algorithm>);
    }

    private int id;
    [Column(Storage = "id", DbType = "integer", IsPrimaryKey = true, IsDbGenerated = true, UpdateCheck = UpdateCheck.Never)]
    public int Id {
      get { return this.id; }
      private set { this.id = value; }
    }

    private object data;
    [Column(Storage = "data", DbType = "text", CanBeNull = false)]
    public object Data {
      get { return this.data; }
      set { this.data = value; }
    }

    private int algorithmId;
    [Column(Storage="algorithmId", DbType="Int not null")]
    public int AlgorithmId {
      get { return this.algorithmId;}
      private set { 
        if(algorithmId != value) {
          if(algorithm.HasLoadedOrAssignedValue)
            throw new ForeignKeyReferenceAlreadyHasValueException();
          algorithmId =value;
        }
      }
    }


    private EntityRef<Algorithm> algorithm;
    [Association(Storage="algorithm",ThisKey="AlgorithmId",OtherKey="Id",IsForeignKey=true)]
    public Algorithm Algorithm {
      get { return this.algorithm.Entity; }
      set {
        Algorithm previousValue = algorithm.Entity;
        if (previousValue != value || (!algorithm.HasLoadedOrAssignedValue)) {
          if (previousValue == null) {
            algorithm.Entity = null;
            previousValue.Models.Remove(this);
          }
          algorithm.Entity = value;
          if (value != null) {
            value.Models.Add(this);
            algorithmId = value.Id;
          } else
            throw new NullReferenceException("Null not allowed as value for Model.Algorithm");
        }
      }
    }

    private int targetVariableId;
    [Column(Storage = "targetVariableId", DbType = "Int not null")]
    public int TargetVariableId {
      get { return this.targetVariableId; }
      private set {
        if (targetVariableId != value) {
          if (targetVariable.HasLoadedOrAssignedValue)
            throw new ForeignKeyReferenceAlreadyHasValueException();
          targetVariableId = value;
        }
      }
    }

    private EntityRef<Variable> targetVariable;
    [Association(Storage = "targetVariable", ThisKey = "TargetVariableId", OtherKey = "Id", IsForeignKey = true)]
    public Variable TargetVariable {
      get { return this.targetVariable.Entity; }
      set {
        Variable previousValue = targetVariable.Entity;
        if (previousValue != value || (!targetVariable.HasLoadedOrAssignedValue)) {
          if (previousValue == null) {
            targetVariable.Entity = null;
            previousValue.ModelsPredictingThisVariable.Remove(this);
          }
          targetVariable.Entity = value;
          if (value != null) {
            value.ModelsPredictingThisVariable.Add(this);
            targetVariableId = value.Id;
          } else
            throw new NullReferenceException("Null not allowed as value for Model.TargetVariable");
        }
      }
    }

    public IEnumerable<Variable> inputVariables {
      get {
        using (ModelingDataContext ctx = new ModelingDataContext()) {
          var x = from inputVariable in ctx.InputVariables
                  where inputVariable.Model == this
                  select inputVariable.Variable;
          return x.AsEnumerable();
        }
      }
    }
  }
}
